'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.loader = loader;
exports.render = render;

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

var _loaderUtils = require('loader-utils');

var LoaderUtils = _interopRequireWildcard(_loaderUtils);

var _babelTypes = require('babel-types');

var _babelGenerator = require('babel-generator');

var _babelGenerator2 = _interopRequireDefault(_babelGenerator);

var _postcss = require('postcss');

var postcss = _interopRequireWildcard(_postcss);

var _postcssSelectorParser = require('postcss-selector-parser');

var _postcssSelectorParser2 = _interopRequireDefault(_postcssSelectorParser);

var _HTMLTagList = require('./HTMLTagList');

var _HTMLTagList2 = _interopRequireDefault(_HTMLTagList);

var _ComponentRef = require('./ComponentRef');

var ComponentRef = _interopRequireWildcard(_ComponentRef);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @copyright 2016-present, React CSS Components team
 * 
 */

var LOADER = require.resolve('../webpack');

var COMPONENT_RE = /^[a-zA-Z_0-9]+$/;

function findComponentNames(node) {
  var componentNames = [];
  (0, _postcssSelectorParser2.default)().process(node.selector).res.eachTag(function (selector) {
    if (COMPONENT_RE.exec(selector.value)) {
      componentNames.push(selector.value);
    }
  });
  return componentNames;
}

function renderToCSS(source) {
  var root = postcss.parse(source);
  root.walkRules(function (node) {
    removeBaseDeclaration(node);
    localizeComponentRule(node);
  });
  return root.toString();
}

function removeBaseDeclaration(node) {
  node.walkDecls(function (node) {
    if (node.prop === 'base') {
      node.remove();
    }
  });
}

function localizeComponentRule(node) {
  var componentNames = findComponentNames(node);
  if (componentNames.length > 0) {
    (function () {
      var toProcess = [];
      var selector = (0, _postcssSelectorParser2.default)().process(node.selector).res;
      selector.eachTag(function (selector) {
        if (componentNames.indexOf(selector.value) > -1) {
          toProcess.push(selector);
        }
      });
      toProcess.forEach(function (selector) {
        var nextSelector = (0, _postcssSelectorParser.className)({ value: selector.value });
        selector.replaceWith(nextSelector);
      });
      var nextNode = node.clone();
      nextNode.selector = selector.toString();
      node.replaceWith(nextNode);
    })();
  }
}

function renderToJS(source, config) {
  var root = postcss.parse(source);

  var imports = function (_param) {
    return [{
      "type": "ImportDeclaration",
      "specifiers": [{
        "type": "ImportDefaultSpecifier",
        "local": {
          "type": "Identifier",
          "name": "React"
        }
      }],
      "importKind": "value",
      "source": {
        "type": "StringLiteral",
        "extra": {
          "rawValue": "react",
          "raw": "\"react\""
        },
        "value": "react"
      }
    }, {
      "type": "ImportDeclaration",
      "specifiers": [{
        "type": "ImportDefaultSpecifier",
        "local": {
          "type": "Identifier",
          "name": "styles"
        }
      }],
      "importKind": "value",
      "source": _param
    }];
  }((0, _babelTypes.stringLiteral)(config.requestCSS));
  var statements = [];
  var components = {};

  function registerComponent(componentName) {
    if (components[componentName] === undefined) {
      components[componentName] = { base: (0, _babelTypes.stringLiteral)('div') };
    }
  }

  function configureComponentBase(componentName, base) {
    (0, _invariant2.default)(components[componentName], 'Trying to configure base for an unknown component %s', componentName);

    if (_HTMLTagList2.default[base]) {
      base = (0, _babelTypes.stringLiteral)(base);
    } else {
      var ref = ComponentRef.parse(base);
      (0, _invariant2.default)(ref != null, 'Found invalid component ref: %s', base);
      base = (0, _babelTypes.identifier)(componentName + '__Base');
      imports.push(function (_param2, _param3, _param4) {
        return {
          "type": "ImportDeclaration",
          "specifiers": [{
            "type": "ImportSpecifier",
            "imported": _param2,
            "local": _param3
          }],
          "importKind": "value",
          "source": _param4
        };
      }((0, _babelTypes.identifier)(ref.name), base, (0, _babelTypes.stringLiteral)(ref.source)));
    }

    components[componentName].base = base;
  }

  // walk CSS AST and register all component configurations
  root.walkRules(function (node) {
    var componentNames = findComponentNames(node);
    if (componentNames.length === 0) {
      return;
    }
    componentNames.forEach(function (componentName) {
      registerComponent(componentName);
    });
    if (componentNames.length === 1) {
      (function () {
        var componentName = componentNames[0];
        node.walkDecls(function (decl) {
          if (decl.prop === 'base') {
            configureComponentBase(componentName, decl.value);
          }
        });
      })();
    }
  });

  // generate JS code from component configurations
  for (var componentName in components) {
    var component = components[componentName];
    if (components.hasOwnProperty(componentName)) {
      statements.push(function (_param5, _param6, _param7) {
        return {
          "type": "ExportNamedDeclaration",
          "specifiers": [],
          "source": null,
          "declaration": {
            "type": "FunctionDeclaration",
            "id": _param5,
            "generator": false,
            "expression": false,
            "async": false,
            "params": [{
              "type": "Identifier",
              "name": "props"
            }],
            "body": {
              "type": "BlockStatement",
              "body": [{
                "type": "ReturnStatement",
                "argument": {
                  "type": "CallExpression",
                  "callee": {
                    "type": "MemberExpression",
                    "object": {
                      "type": "Identifier",
                      "name": "React"
                    },
                    "property": {
                      "type": "Identifier",
                      "name": "createElement"
                    },
                    "computed": false
                  },
                  "arguments": [_param6, {
                    "type": "ObjectExpression",
                    "properties": [{
                      "type": "SpreadProperty",
                      "argument": {
                        "type": "Identifier",
                        "name": "props"
                      }
                    }, {
                      "type": "ObjectProperty",
                      "method": false,
                      "shorthand": false,
                      "computed": false,
                      "key": {
                        "type": "Identifier",
                        "name": "className"
                      },
                      "value": {
                        "type": "MemberExpression",
                        "object": {
                          "type": "Identifier",
                          "name": "styles"
                        },
                        "property": _param7,
                        "computed": false
                      }
                    }]
                  }]
                }
              }],
              "directives": []
            }
          },
          "exportKind": "value"
        };
      }((0, _babelTypes.identifier)(componentName), component.base, (0, _babelTypes.identifier)(componentName)));
    }
  }

  return (0, _babelGenerator2.default)((0, _babelTypes.program)(imports.concat(statements))).code;
}

/**
 * Webpack loader for React CSS component modules.
 */
function loader(source) {
  this.cacheable();
  var query = LoaderUtils.parseQuery(this.query);
  if (query.css) {
    var result = renderToCSS(source);
    return result;
  } else {
    var _requestCSS = '!!style-loader!css-loader?modules!' + LOADER + '?css!' + this.resource;
    var _result = renderToJS(source, { requestCSS: _requestCSS });
    return _result;
  }
}

/**
 * Render React CSS component module into JS and CSS sources.
 */
function render(source, config) {
  var js = renderToJS(source, { requestCSS: config.requestCSS });
  var css = renderToCSS(source);
  return { js: js, css: css };
}
